"use strict";

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _universalWebsocketClient = require("universal-websocket-client");

var _universalWebsocketClient2 = _interopRequireDefault(_universalWebsocketClient);

var _utilities = require("../utilities");

var _events = require("./events");

var events = _interopRequireWildcard(_events);

var _NatterlyReactor = require("./NatterlyReactor");

var _NatterlyReactor2 = _interopRequireDefault(_NatterlyReactor);

var _NatterlyChat = require("./NatterlyChat");

var _NatterlyChat2 = _interopRequireDefault(_NatterlyChat);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var NatterlyClient = function () {
  function NatterlyClient(siteToken) {
    var _this = this;

    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, NatterlyClient);

    // input by user
    this.siteToken = siteToken;
    this.options = options;
    // If a url for the backend is provided use that
    // else if a protocol and host aare provided form a url from those
    // else use the default cloud natterly api url
    this.url = options.url ? options.url : options.protocol && options.host ? options.protocol + "://" + options.host + "/api/v1" : "https://api.natterly.com/api/v1";
    // internals
    this.websocket = null;
    this.session = null;
    this.visitor = null;
    this._reactor = new _NatterlyReactor2.default();
    this._pingInterval = null;
    // register the websocket events
    this._registerInitialEvents(events);
    // add event listeners
    this.on("VisitorDetailsAdded", function (data) {
      _this.visitor = data;
    });
    this.on("SessionJoined", function (data) {
      _this.visitor = data.session.visitor;
    });
    window.addEventListener("beforeunload", this.disconnect);
  }

  _createClass(NatterlyClient, [{
    key: "connect",
    value: function connect(token) {
      var _this2 = this;

      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var callback = arguments[2];

      var _assignFlexibleCallba = (0, _utilities.assignFlexibleCallbackArgs)(options, callback);

      options = _assignFlexibleCallba.options;
      callback = _assignFlexibleCallba.callback;

      if (options.reconnect && this.websocket && this.websocket.url) {
        this.connectToWebsocket(this.websocket.url, options.page_url);
        return;
      }
      var _options = options,
          page_url = _options.page_url,
          page_title = _options.page_title,
          time_zone = _options.time_zone,
          language = _options.language,
          visitor = _options.visitor,
          custom_data = _options.custom_data,
          browser = _options.browser;

      var site = this.siteToken;
      var requestData = {
        site: site,
        token: token,
        page_url: page_url,
        page_title: page_title,
        language: language,
        time_zone: time_zone,
        browser: browser
      };
      if (visitor) requestData.visitor = visitor;
      if (custom_data) requestData.custom_data = custom_data;
      this.request("sessions_for_sessions/register", { body: requestData }, function (success, data) {
        if (success) {
          _this2.session = data.session;
          _this2.websocketUrl = data.websocket_url;

          // for backwards compatability
          _this2._reactor.dispatchEvent("SessionJoined", data);
          _this2._reactor.dispatchEvent("SiteAvailability", data);

          if (data.ws_connect) {
            _this2.connectToWebsocket(data.websocket_url);
          }
          if (typeof callback === "function") {
            callback(data);
          }
        } else {
          console.error("Error connecting", data);
        }
      });
    }

    // add a new event listener

  }, {
    key: "on",
    value: function on(eventName, callback) {
      this._reactor.addEventListener(eventName, callback);
    }

    // remove an event listener

  }, {
    key: "off",
    value: function off(eventName, callback) {
      this._reactor.removeEventListener(eventName, callback);
    }

    // disconnect from the websocket

  }, {
    key: "disconnect",
    value: function disconnect() {
      clearInterval(this._pingInterval);
      if (this.websocket) {
        if (this.websocket.readyState === 1) {
          this.websocket.close(1000);
        }
        this.websocket = null;
      }
    }

    // send an action to the socket

  }, {
    key: "send",
    value: function send(action) {
      this.websocket.send(action);
    }

    // make an api request

  }, {
    key: "request",
    value: function request(endpoint, data, callback) {
      endpoint = endpoint.replace(/^\//, "");
      var url = this.url.replace(/\/$/, "");
      return (0, _utilities.apiRequest)(url + "/" + endpoint, data, callback);
    }

    // Socket Actions

  }, {
    key: "setPendingMessage",
    value: function setPendingMessage(message) {
      if ((0, _utilities.requiredFields)("setPendingMessage", { message: message })) {
        this.send((0, _utilities.createAction)("C_SetPendingMessage", { message: message }));
      }
    }
  }, {
    key: "markMessageAsSeen",
    value: function markMessageAsSeen(messageId) {
      if ((0, _utilities.requiredFields)("markMessageAsSeen", { id: messageId })) {
        this.send((0, _utilities.createAction)("C_SeeMessage", { message: messageId }));
      }
    }
  }, {
    key: "markPremessageAsSeen",
    value: function markPremessageAsSeen(messageId) {
      if ((0, _utilities.requiredFields)("markMessageAsSeen", { id: messageId })) {
        this.send((0, _utilities.createAction)("C_SeePreMessage", { message: messageId }));
      }
    }
  }, {
    key: "ping",
    value: function ping() {
      this.send((0, _utilities.createAction)("C_Ping"));
    }
  }, {
    key: "connectToWebsocket",
    value: function connectToWebsocket(callback) {
      var _this3 = this;

      var pingInterval = this._pingInterval;
      if (this.websocket) this.disconnect();
      this.websocket = new _universalWebsocketClient2.default(this.websocketUrl);
      // Websocket events
      this.websocket.onmessage = function (message) {
        return _this3._handleEvent(message);
      };
      this.websocket.onclose = function () {
        return clearInterval(pingInterval);
      };
      this.websocket.onopen = function () {
        _this3._joinSession(_this3.session, window.location.href);
        _this3.ping();
        _this3._pingInterval = setInterval(function () {
          return _this3.ping();
        }, 20000);
        if (typeof callback === "function") {
          callback();
        }
      };
    }

    // API Actions

  }, {
    key: "initiateChat",
    value: function initiateChat(subject) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var callback = arguments[2];

      var _assignFlexibleCallba2 = (0, _utilities.assignFlexibleCallbackArgs)(options, callback);

      options = _assignFlexibleCallba2.options;
      callback = _assignFlexibleCallba2.callback;
      var _options2 = options,
          departmentID = _options2.departmentID;

      var body = {
        subject: subject,
        department_id: departmentID
      };
      if ((0, _utilities.requiredFields)("initiateChat", { subject: subject })) {
        this._actionRequest("chats_for_sessions/initiate", body, callback);
      }
    }
  }, {
    key: "getChat",
    value: function getChat(chat) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var callback = arguments[2];

      var _assignFlexibleCallba3 = (0, _utilities.assignFlexibleCallbackArgs)(options, callback);

      options = _assignFlexibleCallba3.options;
      callback = _assignFlexibleCallba3.callback;

      var body = { chat: chat };
      if ((0, _utilities.requiredFields)("getChat", { chat: chat })) {
        this._actionRequest("chats_for_sessions/chat", body, callback);
      }
    }
  }, {
    key: "setVisitorDetails",
    value: function setVisitorDetails() {
      var details = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var callback = arguments[1];

      // form the correct request shape
      var emailAddress = details.emailAddress;

      var visitor = _extends({
        email_address: emailAddress
      }, details);
      delete visitor.emailAddress;
      if ((0, _utilities.requiredFields)("setVisitorDetails", {
        email_address: visitor.email_address,
        name: visitor.name
      })) {
        this._actionRequest("sessions_for_sessions/set_visitor_details", { visitor: visitor }, callback);
      }
    }

    // Private methods

  }, {
    key: "_joinSession",
    value: function _joinSession(token, pageUrl) {
      if ((0, _utilities.requiredFields)("joinSession", { token: token })) {
        this.send((0, _utilities.createAction)("C_JoinSession", { token: token, url: pageUrl }));
      }
    }
  }, {
    key: "_registerInitialEvents",
    value: function _registerInitialEvents(events) {
      for (var key in events) {
        var eventName = events[key];
        this._reactor.registerEvent(eventName);
      }
    }
  }, {
    key: "_handleEvent",
    value: function _handleEvent(message) {
      var data = JSON.parse(message.data);
      var eventName = data.event;
      switch (eventName) {
        case "ChatInitiated":
        case "RestoreChat":
          var chat = new _NatterlyChat2.default(data.payload, this);
          this._reactor.dispatchEvent(eventName, chat);
          break;
        default:
          this._reactor.dispatchEvent(eventName, data.payload);
      }
    }
  }, {
    key: "_actionRequest",
    value: function _actionRequest(endpoint, body, callback) {
      var options = {
        headers: { "X-Session-ID": this.session },
        body: body
      };
      this.request(endpoint, options, function (success, data) {
        if (typeof callback === "function") {
          callback(success, data);
        }
      });
    }
  }]);

  return NatterlyClient;
}();

module.exports = NatterlyClient;